@@ -1,3 +1,18 @@
+/*
+© 2020 Red Hat, Inc. and others
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+*/
 package controller
 
 import (
@@ -5,7 +20,9 @@ import (
 	"reflect"
 
 	"github.com/pkg/errors"
+	"github.com/prometheus/client_golang/prometheus"
 	"github.com/submariner-io/admiral/pkg/log"
+	"github.com/submariner-io/admiral/pkg/resource"
 	"github.com/submariner-io/admiral/pkg/syncer"
 	"github.com/submariner-io/admiral/pkg/syncer/broker"
 	"github.com/submariner-io/admiral/pkg/util"
@@ -13,57 +30,32 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	discovery "k8s.io/api/discovery/v1beta1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
-	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
-	"k8s.io/client-go/dynamic"
 	"k8s.io/client-go/kubernetes"
-	"k8s.io/client-go/kubernetes/scheme"
-	"k8s.io/client-go/rest"
 	"k8s.io/client-go/util/retry"
 	"k8s.io/klog"
 	mcsv1a1 "sigs.k8s.io/mcs-api/pkg/apis/v1alpha1"
 )
 
 const (
-	submarinerIpamGlobalIp = "submariner.io/globalIp"
+	submarinerIPAMGlobalIP = "submariner.io/globalIp"
 	serviceUnavailable     = "ServiceUnavailable"
 	invalidServiceType     = "UnsupportedServiceType"
 	clusterIP              = "cluster-ip"
 )
 
-var MaxExportStatusConditions = 10
-
-func New(spec *AgentSpecification, cfg *rest.Config) (*Controller, error) {
-	kubeClientSet, err := kubernetes.NewForConfig(cfg)
-	if err != nil {
-		return nil, fmt.Errorf("Error building clientset: %s", err.Error())
-	}
-
-	syncerConf := &broker.SyncerConfig{LocalRestConfig: cfg}
-
-	restMapper, err := util.BuildRestMapper(cfg)
-	if err != nil {
-		return nil, err
-	}
-
-	localClient, err := dynamic.NewForConfig(cfg)
-	if err != nil {
-		return nil, fmt.Errorf("error creating dynamic client: %v", err)
-	}
-
-	return NewWithDetail(spec, syncerConf, restMapper, localClient, kubeClientSet, scheme.Scheme,
-		func(config *broker.SyncerConfig) (*broker.Syncer, error) {
-			return broker.NewSyncer(*config)
-		})
+type AgentConfig struct {
+	ServiceImportCounterName string
+	ServiceExportCounterName string
 }
 
-// Constructor that takes additional detail. This is intended for unit tests.
-func NewWithDetail(spec *AgentSpecification, syncerConf *broker.SyncerConfig, restMapper meta.RESTMapper, localClient dynamic.Interface,
-	kubeClientSet kubernetes.Interface, runtimeScheme *runtime.Scheme,
-	newSyncer func(*broker.SyncerConfig) (*broker.Syncer, error)) (*Controller, error) {
+var MaxExportStatusConditions = 10
+
+func New(spec *AgentSpecification, syncerConf broker.SyncerConfig, kubeClientSet kubernetes.Interface,
+	syncerMetricNames AgentConfig) (*Controller, error) {
 	agentController := &Controller{
 		clusterID:        spec.ClusterID,
 		namespace:        spec.Namespace,
@@ -71,14 +63,13 @@ func NewWithDetail(spec *AgentSpecification, syncerConf *broker.SyncerConfig, re
 		kubeClientSet:    kubeClientSet,
 	}
 
-	_, gvr, err := util.ToUnstructuredResource(&mcsv1a1.ServiceExport{}, restMapper)
+	_, gvr, err := util.ToUnstructuredResource(&mcsv1a1.ServiceExport{}, syncerConf.RestMapper)
 	if err != nil {
 		return nil, err
 	}
 
-	agentController.serviceExportClient = localClient.Resource(*gvr)
+	agentController.serviceExportClient = syncerConf.LocalClient.Resource(*gvr)
 
-	syncerConf.Scheme = runtimeScheme
 	syncerConf.LocalNamespace = spec.Namespace
 	syncerConf.LocalClusterID = spec.ClusterID
 
@@ -87,9 +78,14 @@ func NewWithDetail(spec *AgentSpecification, syncerConf *broker.SyncerConfig, re
 			LocalSourceNamespace: metav1.NamespaceAll,
 			LocalResourceType:    &mcsv1a1.ServiceImport{},
 			BrokerResourceType:   &mcsv1a1.ServiceImport{},
+			SyncCounterOpts: &prometheus.GaugeOpts{
+				Name: syncerMetricNames.ServiceImportCounterName,
+				Help: "Count of imported services",
+			},
 		},
 	}
-	agentController.serviceImportSyncer, err = newSyncer(syncerConf)
+
+	agentController.serviceImportSyncer, err = broker.NewSyncer(syncerConf)
 	if err != nil {
 		return nil, err
 	}
@@ -110,22 +106,26 @@ func NewWithDetail(spec *AgentSpecification, syncerConf *broker.SyncerConfig, re
 			BrokerTransform: agentController.remoteEndpointSliceToLocal,
 		},
 	}
-	agentController.endpointSliceSyncer, err = newSyncer(syncerConf)
+
+	agentController.endpointSliceSyncer, err = broker.NewSyncer(syncerConf)
 	if err != nil {
 		return nil, err
 	}
 
 	agentController.serviceExportSyncer, err = syncer.NewResourceSyncer(&syncer.ResourceSyncerConfig{
 		Name:             "ServiceExport -> ServiceImport",
-		SourceClient:     localClient,
+		SourceClient:     syncerConf.LocalClient,
 		SourceNamespace:  metav1.NamespaceAll,
-		Direction:        syncer.RemoteToLocal,
-		RestMapper:       restMapper,
+		RestMapper:       syncerConf.RestMapper,
 		Federator:        agentController.serviceImportSyncer.GetLocalFederator(),
 		ResourceType:     &mcsv1a1.ServiceExport{},
 		Transform:        agentController.serviceExportToServiceImport,
 		OnSuccessfulSync: agentController.onSuccessfulServiceImportSync,
-		Scheme:           runtimeScheme,
+		Scheme:           syncerConf.Scheme,
+		SyncCounterOpts: &prometheus.GaugeOpts{
+			Name: syncerMetricNames.ServiceExportCounterName,
+			Help: "Count of exported services",
+		},
 	})
 	if err != nil {
 		return nil, err
@@ -133,26 +133,20 @@ func NewWithDetail(spec *AgentSpecification, syncerConf *broker.SyncerConfig, re
 
 	agentController.serviceSyncer, err = syncer.NewResourceSyncer(&syncer.ResourceSyncerConfig{
 		Name:            "Service deletion",
-		SourceClient:    localClient,
+		SourceClient:    syncerConf.LocalClient,
 		SourceNamespace: metav1.NamespaceAll,
-		Direction:       syncer.RemoteToLocal,
-		RestMapper:      restMapper,
+		RestMapper:      syncerConf.RestMapper,
 		Federator:       agentController.serviceImportSyncer.GetLocalFederator(),
 		ResourceType:    &corev1.Service{},
 		Transform:       agentController.serviceToRemoteServiceImport,
-		Scheme:          runtimeScheme,
+		Scheme:          syncerConf.Scheme,
 	})
 	if err != nil {
 		return nil, err
 	}
 
-	agentController.serviceImportController, err = newServiceImportController(spec, agentController.serviceSyncer, restMapper,
-		localClient, runtimeScheme)
-	if err != nil {
-		return nil, err
-	}
-
-	agentController.lhServiceExportController, err = newLHServiceExportController(localClient, restMapper, runtimeScheme)
+	agentController.serviceImportController, err = newServiceImportController(spec, agentController.serviceSyncer,
+		syncerConf.RestMapper, syncerConf.LocalClient, syncerConf.Scheme)
 	if err != nil {
 		return nil, err
 	}
@@ -186,16 +180,52 @@ func (a *Controller) Start(stopCh <-chan struct{}) error {
 		return err
 	}
 
-	if err := a.lhServiceExportController.start(stopCh); err != nil {
-		return err
-	}
+	a.serviceExportSyncer.Reconcile(func() []runtime.Object {
+		return a.serviceImportLister(func(si *mcsv1a1.ServiceImport) runtime.Object {
+			return &mcsv1a1.ServiceExport{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      si.GetAnnotations()[lhconstants.OriginName],
+					Namespace: si.GetAnnotations()[lhconstants.OriginNamespace],
+				},
+			}
+		})
+	})
+
+	a.serviceSyncer.Reconcile(func() []runtime.Object {
+		return a.serviceImportLister(func(si *mcsv1a1.ServiceImport) runtime.Object {
+			return &corev1.Service{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      si.GetAnnotations()[lhconstants.OriginName],
+					Namespace: si.GetAnnotations()[lhconstants.OriginNamespace],
+				},
+			}
+		})
+	})
 
 	klog.Info("Agent controller started")
 
 	return nil
 }
 
-func (a *Controller) serviceExportToServiceImport(obj runtime.Object, op syncer.Operation) (runtime.Object, bool) {
+func (a *Controller) serviceImportLister(transform func(si *mcsv1a1.ServiceImport) runtime.Object) []runtime.Object {
+	siList, err := a.serviceImportSyncer.ListLocalResources(&mcsv1a1.ServiceImport{})
+	if err != nil {
+		klog.Errorf("Error listing serviceImports: %v", err)
+		return nil
+	}
+
+	retList := make([]runtime.Object, 0, len(siList))
+
+	for _, obj := range siList {
+		si := obj.(*mcsv1a1.ServiceImport)
+
+		retList = append(retList, transform(si))
+	}
+
+	return retList
+}
+
+func (a *Controller) serviceExportToServiceImport(obj runtime.Object, numRequeues int, op syncer.Operation) (runtime.Object, bool) {
 	svcExport := obj.(*mcsv1a1.ServiceExport)
 
 	if op == syncer.Delete {
@@ -244,7 +274,7 @@ func (a *Controller) serviceExportToServiceImport(obj runtime.Object, op syncer.
 		return nil, false
 	}
 
-	if a.globalnetEnabled && getGlobalIpFromService(svc) == "" {
+	if a.globalnetEnabled && getGlobalIPFromService(svc) == "" {
 		klog.V(log.DEBUG).Infof("Service to be exported (%s/%s) doesn't have a global IP yet", svcExport.Namespace, svcExport.Name)
 
 		// Globalnet enabled but service doesn't have globalIp yet, Update the status and requeue
@@ -328,7 +358,7 @@ func (a *Controller) onSuccessfulServiceImportSync(synced runtime.Object, op syn
 		"", "Service was successfully synced to the broker")
 }
 
-func (a *Controller) serviceToRemoteServiceImport(obj runtime.Object, op syncer.Operation) (runtime.Object, bool) {
+func (a *Controller) serviceToRemoteServiceImport(obj runtime.Object, numRequeues int, op syncer.Operation) (runtime.Object, bool) {
 	if op != syncer.Delete {
 		// Ignore create/update
 		return nil, false
@@ -396,7 +426,7 @@ func (a *Controller) updateExportedServiceStatus(name, namespace string, condTyp
 			toUpdate.Status.Conditions = append(toUpdate.Status.Conditions, exportCondition)
 		}
 
-		raw, err := util.ToUnstructured(toUpdate)
+		raw, err := resource.ToUnstructured(toUpdate)
 		if err != nil {
 			return err
 		}
@@ -433,7 +463,7 @@ func serviceExportConditionEqual(c1, c2 *mcsv1a1.ServiceExportCondition) bool {
 func (a *Controller) newServiceImport(svcExport *mcsv1a1.ServiceExport) *mcsv1a1.ServiceImport {
 	return &mcsv1a1.ServiceImport{
 		ObjectMeta: metav1.ObjectMeta{
-			Name: a.getObjectNameWithClusterId(svcExport.Name, svcExport.Namespace),
+			Name: a.getObjectNameWithClusterID(svcExport.Name, svcExport.Namespace),
 			Annotations: map[string]string{
 				lhconstants.OriginName:      svcExport.Name,
 				lhconstants.OriginNamespace: svcExport.Namespace,
@@ -449,20 +479,23 @@ func (a *Controller) newServiceImport(svcExport *mcsv1a1.ServiceExport) *mcsv1a1
 
 func (a *Controller) getIPsAndPortsForService(service *corev1.Service, siType mcsv1a1.ServiceImportType) (
 	[]string, []mcsv1a1.ServicePort, error) {
-	mcsPort := mcsv1a1.ServicePort{}
-	if len(service.Spec.Ports) > 0 {
-		mcsPort.Name = service.Spec.Ports[0].Name
-		mcsPort.Protocol = service.Spec.Ports[0].Protocol
-		mcsPort.Port = service.Spec.Ports[0].Port
+	var mcsPorts = make([]mcsv1a1.ServicePort, 0, len(service.Spec.Ports))
+
+	for _, port := range service.Spec.Ports {
+		mcsPorts = append(mcsPorts, mcsv1a1.ServicePort{
+			Name:     port.Name,
+			Protocol: port.Protocol,
+			Port:     port.Port,
+		})
 	}
 
 	if siType == mcsv1a1.ClusterSetIP {
-		mcsIp := getGlobalIpFromService(service)
-		if mcsIp == "" {
-			mcsIp = service.Spec.ClusterIP
+		mcsIP := getGlobalIPFromService(service)
+		if mcsIP == "" {
+			mcsIP = service.Spec.ClusterIP
 		}
 
-		return []string{mcsIp}, []mcsv1a1.ServicePort{mcsPort}, nil
+		return []string{mcsIP}, mcsPorts, nil
 	}
 
 	endpoint, err := a.kubeClientSet.CoreV1().Endpoints(service.Namespace).Get(service.Name, metav1.GetOptions{})
@@ -472,13 +505,13 @@ func (a *Controller) getIPsAndPortsForService(service *corev1.Service, siType mc
 			return nil, nil, errors.WithMessage(err, "Error retrieving the Endpoints for the Service")
 		}
 
-		return make([]string, 0), make([]mcsv1a1.ServicePort, 0), nil
+		return make([]string, 0), nil, nil
 	}
 
-	return getIPsFromEndpoint(endpoint), []mcsv1a1.ServicePort{mcsPort}, nil
+	return getIPsFromEndpoint(endpoint), mcsPorts, nil
 }
 
-func (a *Controller) getObjectNameWithClusterId(name, namespace string) string {
+func (a *Controller) getObjectNameWithClusterID(name, namespace string) string {
 	return name + "-" + namespace + "-" + a.clusterID
 }
 
@@ -496,25 +529,25 @@ func getIPsFromEndpoint(endpoint *corev1.Endpoints) []string {
 	return ipList
 }
 
-func getGlobalIpFromService(service *corev1.Service) string {
+func getGlobalIPFromService(service *corev1.Service) string {
 	if service != nil {
 		annotations := service.GetAnnotations()
 		if annotations != nil {
-			return annotations[submarinerIpamGlobalIp]
+			return annotations[submarinerIPAMGlobalIP]
 		}
 	}
 
 	return ""
 }
 
-func (a *Controller) remoteEndpointSliceToLocal(obj runtime.Object, op syncer.Operation) (runtime.Object, bool) {
+func (a *Controller) remoteEndpointSliceToLocal(obj runtime.Object, numRequeues int, op syncer.Operation) (runtime.Object, bool) {
 	endpointSlice := obj.(*discovery.EndpointSlice)
 	endpointSlice.Namespace = endpointSlice.GetObjectMeta().GetLabels()[lhconstants.LabelSourceNamespace]
 
 	return endpointSlice, false
 }
 
-func (a *Controller) filterLocalEndpointSlices(obj runtime.Object, op syncer.Operation) (runtime.Object, bool) {
+func (a *Controller) filterLocalEndpointSlices(obj runtime.Object, numRequeues int, op syncer.Operation) (runtime.Object, bool) {
 	endpointSlice := obj.(*discovery.EndpointSlice)
 	labels := endpointSlice.GetObjectMeta().GetLabels()
 
