@@ -1,8 +1,25 @@
+/*
+© 2020 Red Hat, Inc. and others
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+*/
 package controller_test
 
 import (
+	"crypto/rand"
 	"errors"
 	"fmt"
+	"math/big"
 	"reflect"
 	"sort"
 	"time"
@@ -14,12 +31,10 @@ import (
 	"github.com/submariner-io/admiral/pkg/syncer/broker"
 	"github.com/submariner-io/admiral/pkg/syncer/test"
 	"github.com/submariner-io/lighthouse/pkg/agent/controller"
-	lighthousev2a1 "github.com/submariner-io/lighthouse/pkg/apis/lighthouse.submariner.io/v2alpha1"
 	lhconstants "github.com/submariner-io/lighthouse/pkg/constants"
 	corev1 "k8s.io/api/core/v1"
 	discovery "k8s.io/api/discovery/v1beta1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
-	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
@@ -160,6 +175,161 @@ var _ = Describe("ServiceImport syncing", func() {
 			t.awaitNoServiceImport(t.brokerServiceImportClient)
 		})
 	})
+
+	When("a Service has port information", func() {
+		BeforeEach(func() {
+			t.service.Spec.Ports = []corev1.ServicePort{
+				{
+					Name:     "eth0",
+					Protocol: corev1.ProtocolTCP,
+					Port:     123,
+				},
+				{
+					Name:     "eth1",
+					Protocol: corev1.ProtocolTCP,
+					Port:     1234,
+				},
+			}
+		})
+
+		It("should set the appropriate port information in the ServiceImport", func() {
+			t.createService()
+			t.createServiceExport()
+			t.awaitServiceExported(t.service.Spec.ClusterIP, 0)
+		})
+	})
+})
+
+var _ = Describe("Reconciliation", func() {
+	var t *testDriver
+
+	BeforeEach(func() {
+		t = newTestDiver()
+	})
+
+	JustBeforeEach(func() {
+		t.justBeforeEach()
+		t.createService()
+		t.createServiceExport()
+	})
+
+	AfterEach(func() {
+		t.afterEach()
+	})
+
+	When("a local headless ServiceImport is stale on startup due to a missed ServiceExport delete event", func() {
+		BeforeEach(func() {
+			t.service.Spec.ClusterIP = corev1.ClusterIPNone
+		})
+
+		JustBeforeEach(func() {
+			t.createEndpoints()
+		})
+
+		It("should delete the ServiceImport and EndpointSlice on reconciliation", func() {
+			serviceImport := t.cluster1.awaitServiceImport(t.service, mcsv1a1.Headless, "")
+			endpointSlice := t.cluster1.awaitEndpointSlice(t.endpoints, t.service)
+
+			t.afterEach()
+			t = newTestDiver()
+
+			test.CreateResource(t.cluster1.localServiceImportClient, serviceImport)
+			test.CreateResource(t.cluster1.localEndpointSliceClient, endpointSlice)
+			t.createService()
+			t.cluster1.start(t, *t.syncerConfig)
+
+			t.awaitServiceUnexported()
+			t.awaitNoEndpointSlice(t.cluster1.localEndpointSliceClient)
+		})
+	})
+
+	When("a local ServiceImport is stale on startup due to a missed Service delete event", func() {
+		It("should delete the ServiceImport on reconciliation", func() {
+			serviceImport := t.cluster1.awaitServiceImport(t.service, mcsv1a1.ClusterSetIP, t.service.Spec.ClusterIP)
+
+			t.afterEach()
+			t = newTestDiver()
+
+			test.CreateResource(t.cluster1.localServiceImportClient, serviceImport)
+			t.createServiceExport()
+			t.cluster1.start(t, *t.syncerConfig)
+
+			t.awaitServiceUnexported()
+		})
+	})
+
+	When("a synced local ServiceImport is stale in the broker datastore on startup", func() {
+		It("should delete it from the broker datastore on reconciliation", func() {
+			serviceImport := t.awaitBrokerServiceImport(mcsv1a1.ClusterSetIP, t.service.Spec.ClusterIP)
+
+			t.afterEach()
+			t = newTestDiver()
+
+			test.CreateResource(t.brokerServiceImportClient, serviceImport)
+			t.justBeforeEach()
+
+			t.awaitServiceUnexported()
+		})
+	})
+
+	When("a synced remote ServiceImport is stale in the local datastore on startup", func() {
+		It("should delete it from the local datastore on reconciliation", func() {
+			serviceImport := t.cluster2.awaitServiceImport(t.service, mcsv1a1.ClusterSetIP, t.service.Spec.ClusterIP)
+
+			t.afterEach()
+			t = newTestDiver()
+
+			test.CreateResource(t.cluster2.localServiceImportClient, serviceImport)
+			t.cluster2.start(t, *t.syncerConfig)
+
+			t.awaitServiceUnexported()
+		})
+	})
+
+	When("a synced local EndpointSlice is stale in the broker datastore on startup", func() {
+		BeforeEach(func() {
+			t.service.Spec.ClusterIP = corev1.ClusterIPNone
+		})
+
+		JustBeforeEach(func() {
+			t.createEndpoints()
+		})
+
+		It("should delete it from the broker datastore on reconciliation", func() {
+			endpointSlice := t.awaitBrokerEndpointSlice()
+
+			t.afterEach()
+			t = newTestDiver()
+
+			test.CreateResource(t.brokerEndpointSliceClient, endpointSlice)
+			t.justBeforeEach()
+
+			t.awaitNoEndpointSlice(t.brokerEndpointSliceClient)
+			t.awaitNoEndpointSlice(t.cluster2.localEndpointSliceClient)
+		})
+	})
+
+	When("a synced remote EndpointSlice is stale in the local datastore on startup", func() {
+		BeforeEach(func() {
+			t.service.Spec.ClusterIP = corev1.ClusterIPNone
+		})
+
+		JustBeforeEach(func() {
+			t.createEndpoints()
+		})
+
+		It("should delete it from the local datastore on reconciliation", func() {
+			endpointSlice := t.cluster2.awaitEndpointSlice(t.endpoints, t.service)
+
+			t.afterEach()
+			t = newTestDiver()
+
+			test.CreateResource(t.cluster2.localEndpointSliceClient, endpointSlice)
+			t.cluster2.start(t, *t.syncerConfig)
+
+			t.awaitNoEndpointSlice(t.cluster2.localEndpointSliceClient)
+		})
+	})
 })
 
 var _ = Describe("Globalnet enabled", func() {
@@ -343,15 +513,13 @@ type cluster struct {
 type testDriver struct {
 	cluster1                  cluster
 	cluster2                  cluster
-	brokerDynClient           dynamic.Interface
 	brokerServiceImportClient *fake.DynamicResourceClient
 	brokerEndpointSliceClient *fake.DynamicResourceClient
 	service                   *corev1.Service
 	serviceExport             *mcsv1a1.ServiceExport
 	endpoints                 *corev1.Endpoints
 	stopCh                    chan struct{}
-	restMapper                meta.RESTMapper
-	syncerScheme              *runtime.Scheme
+	syncerConfig              *broker.SyncerConfig
 }
 
 func newTestDiver() *testDriver {
@@ -359,7 +527,6 @@ func newTestDiver() *testDriver {
 	Expect(corev1.AddToScheme(syncerScheme)).To(Succeed())
 	Expect(discovery.AddToScheme(syncerScheme)).To(Succeed())
 	Expect(mcsv1a1.AddToScheme(syncerScheme)).To(Succeed())
-	Expect(lighthousev2a1.AddToScheme(syncerScheme)).To(Succeed())
 
 	t := &testDriver{
 		cluster1: cluster{
@@ -386,11 +553,14 @@ func newTestDiver() *testDriver {
 				Selector:  map[string]string{"app": "test"},
 			},
 		},
-		restMapper: test.GetRESTMapperFor(&mcsv1a1.ServiceExport{}, &mcsv1a1.ServiceImport{},
-			&corev1.Service{}, &corev1.Endpoints{}, &discovery.EndpointSlice{}, &lighthousev2a1.ServiceExport{}),
-		brokerDynClient: fake.NewDynamicClient(syncerScheme),
-		syncerScheme:    syncerScheme,
-		stopCh:          make(chan struct{}),
+		syncerConfig: &broker.SyncerConfig{
+			BrokerNamespace: test.RemoteNamespace,
+			RestMapper: test.GetRESTMapperFor(&mcsv1a1.ServiceExport{}, &mcsv1a1.ServiceImport{},
+				&corev1.Service{}, &corev1.Endpoints{}, &discovery.EndpointSlice{}),
+			BrokerClient: fake.NewDynamicClient(syncerScheme),
+			Scheme:       syncerScheme,
+		},
+		stopCh: make(chan struct{}),
 	}
 
 	t.serviceExport = &mcsv1a1.ServiceExport{
@@ -434,55 +604,60 @@ func newTestDiver() *testDriver {
 		},
 	}
 
-	t.brokerServiceImportClient = t.brokerDynClient.Resource(*test.GetGroupVersionResourceFor(t.restMapper,
+	t.brokerServiceImportClient = t.syncerConfig.BrokerClient.Resource(*test.GetGroupVersionResourceFor(t.syncerConfig.RestMapper,
 		&mcsv1a1.ServiceImport{})).Namespace(test.RemoteNamespace).(*fake.DynamicResourceClient)
 
-	t.brokerEndpointSliceClient = t.brokerDynClient.Resource(*test.GetGroupVersionResourceFor(t.restMapper,
+	t.brokerEndpointSliceClient = t.syncerConfig.BrokerClient.Resource(*test.GetGroupVersionResourceFor(t.syncerConfig.RestMapper,
 		&discovery.EndpointSlice{})).Namespace(test.RemoteNamespace).(*fake.DynamicResourceClient)
 
-	t.cluster1.init(t.restMapper, syncerScheme)
-	t.cluster2.init(t.restMapper, syncerScheme)
+	t.cluster1.init(*t.syncerConfig)
+	t.cluster2.init(*t.syncerConfig)
 
 	return t
 }
 
 func (t *testDriver) justBeforeEach() {
-	syncerConfig := &broker.SyncerConfig{
-		BrokerNamespace: test.RemoteNamespace,
-	}
-
-	t.cluster1.start(t, syncerConfig, t.syncerScheme)
-	t.cluster2.start(t, syncerConfig, t.syncerScheme)
+	t.cluster1.start(t, *t.syncerConfig)
+	t.cluster2.start(t, *t.syncerConfig)
 }
 
 func (t *testDriver) afterEach() {
 	close(t.stopCh)
 }
 
-func (c *cluster) init(restMapper meta.RESTMapper, syncerScheme *runtime.Scheme) {
-	c.localDynClient = fake.NewDynamicClient(syncerScheme)
+func (c *cluster) init(syncerConfig broker.SyncerConfig) {
+	c.localDynClient = fake.NewDynamicClient(syncerConfig.Scheme)
 
-	c.localServiceExportClient = c.localDynClient.Resource(*test.GetGroupVersionResourceFor(restMapper,
+	c.localServiceExportClient = c.localDynClient.Resource(*test.GetGroupVersionResourceFor(syncerConfig.RestMapper,
 		&mcsv1a1.ServiceExport{})).Namespace(serviceNamespace).(*fake.DynamicResourceClient)
 
-	c.localServiceImportClient = c.localDynClient.Resource(*test.GetGroupVersionResourceFor(restMapper,
+	c.localServiceImportClient = c.localDynClient.Resource(*test.GetGroupVersionResourceFor(syncerConfig.RestMapper,
 		&mcsv1a1.ServiceImport{})).Namespace(test.LocalNamespace).(*fake.DynamicResourceClient)
 
-	c.localEndpointSliceClient = c.localDynClient.Resource(*test.GetGroupVersionResourceFor(restMapper,
+	c.localEndpointSliceClient = c.localDynClient.Resource(*test.GetGroupVersionResourceFor(syncerConfig.RestMapper,
 		&discovery.EndpointSlice{})).Namespace(serviceNamespace)
 
 	fakeCS := fakeKubeClient.NewSimpleClientset()
 	c.endpointsReactor = fake.NewFailingReactorForResource(&fakeCS.Fake, "endpoints")
 	c.localKubeClient = fakeCS
-
-	fake.AddDeleteCollectionReactor(&fakeCS.Fake, "EndpointSlice")
 }
 
-func (c *cluster) start(t *testDriver, syncerConfig *broker.SyncerConfig, syncerScheme *runtime.Scheme) {
-	agentController, err := controller.NewWithDetail(&c.agentSpec, syncerConfig, t.restMapper, c.localDynClient, c.localKubeClient,
-		syncerScheme, func(config *broker.SyncerConfig) (*broker.Syncer, error) {
-			return broker.NewSyncerWithDetail(config, c.localDynClient, t.brokerDynClient, t.restMapper)
-		})
+func (c *cluster) start(t *testDriver, syncerConfig broker.SyncerConfig) {
+	syncerConfig.LocalClient = c.localDynClient
+	bigint, err := rand.Int(rand.Reader, big.NewInt(1000000))
+	Expect(err).To(Succeed())
+
+	serviceImportCounterName := "submariner_service_import" + bigint.String()
+
+	bigint, err = rand.Int(rand.Reader, big.NewInt(1000000))
+	Expect(err).To(Succeed())
+
+	serviceExportCounterName := "submariner_service_export" + bigint.String()
+
+	agentController, err := controller.New(&c.agentSpec, syncerConfig, c.localKubeClient,
+		controller.AgentConfig{
+			ServiceImportCounterName: serviceImportCounterName,
+			ServiceExportCounterName: serviceExportCounterName})
 
 	Expect(err).To(Succeed())
 	Expect(agentController.Start(t.stopCh)).To(Succeed())
@@ -508,6 +683,14 @@ func awaitServiceImport(client dynamic.ResourceInterface, service *corev1.Servic
 		Expect(serviceImport.Spec.IPs).To(Equal([]string{serviceIP}))
 	}
 
+	Expect(serviceImport.Spec.Ports).To(HaveLen(len(service.Spec.Ports)))
+
+	for i := range service.Spec.Ports {
+		Expect(serviceImport.Spec.Ports[i].Name).To(Equal(service.Spec.Ports[i].Name))
+		Expect(serviceImport.Spec.Ports[i].Protocol).To(Equal(service.Spec.Ports[i].Protocol))
+		Expect(serviceImport.Spec.Ports[i].Port).To(Equal(service.Spec.Ports[i].Port))
+	}
+
 	labels := serviceImport.GetObjectMeta().GetLabels()
 	Expect(labels[lhconstants.LabelSourceNamespace]).To(Equal(service.GetNamespace()))
 	Expect(labels[lhconstants.LabelSourceName]).To(Equal(service.GetName()))
@@ -516,8 +699,8 @@ func awaitServiceImport(client dynamic.ResourceInterface, service *corev1.Servic
 	return serviceImport
 }
 
-func (c *cluster) awaitServiceImport(service *corev1.Service, sType mcsv1a1.ServiceImportType, serviceIP string) {
-	awaitServiceImport(c.localServiceImportClient, service, sType, serviceIP)
+func (c *cluster) awaitServiceImport(service *corev1.Service, sType mcsv1a1.ServiceImportType, serviceIP string) *mcsv1a1.ServiceImport {
+	return awaitServiceImport(c.localServiceImportClient, service, sType, serviceIP)
 }
 
 func awaitUpdatedServiceImport(client dynamic.ResourceInterface, service *corev1.Service,
@@ -558,7 +741,7 @@ func (c *cluster) awaitUpdatedServiceImport(service *corev1.Service, serviceIP s
 }
 
 func awaitEndpointSlice(endpointSliceClient, serviceImportClient dynamic.ResourceInterface,
-	endpoints *corev1.Endpoints, service *corev1.Service, namespace string, expectOwnerRef bool) {
+	endpoints *corev1.Endpoints, service *corev1.Service, namespace string, expectOwnerRef bool) *discovery.EndpointSlice {
 	obj := test.AwaitResource(endpointSliceClient, endpoints.Name+"-"+clusterID1)
 
 	endpointSlice := &discovery.EndpointSlice{}
@@ -622,10 +805,12 @@ func awaitEndpointSlice(endpointSliceClient, serviceImportClient dynamic.Resourc
 		Protocol: &protocol,
 		Port:     &port,
 	}))
+
+	return endpointSlice
 }
 
-func (c *cluster) awaitEndpointSlice(endpoints *corev1.Endpoints, service *corev1.Service) {
-	awaitEndpointSlice(c.localEndpointSliceClient, c.localServiceImportClient, endpoints, service,
+func (c *cluster) awaitEndpointSlice(endpoints *corev1.Endpoints, service *corev1.Service) *discovery.EndpointSlice {
+	return awaitEndpointSlice(c.localEndpointSliceClient, c.localServiceImportClient, endpoints, service,
 		service.Namespace, c.agentSpec.ClusterID == clusterID1)
 }
 
@@ -664,8 +849,12 @@ func (c *cluster) awaitUpdatedEndpointSlice(endpoints *corev1.Endpoints, expecte
 	awaitUpdatedEndpointSlice(c.localEndpointSliceClient, endpoints, expectedIPs)
 }
 
-func (t *testDriver) awaitBrokerServiceImport(sType mcsv1a1.ServiceImportType, serviceIP string) {
-	awaitServiceImport(t.brokerServiceImportClient, t.service, sType, serviceIP)
+func (t *testDriver) awaitBrokerServiceImport(sType mcsv1a1.ServiceImportType, serviceIP string) *mcsv1a1.ServiceImport {
+	return awaitServiceImport(t.brokerServiceImportClient, t.service, sType, serviceIP)
+}
+
+func (t *testDriver) awaitBrokerEndpointSlice() *discovery.EndpointSlice {
+	return awaitEndpointSlice(t.brokerEndpointSliceClient, t.brokerServiceImportClient, t.endpoints, t.service, test.RemoteNamespace, false)
 }
 
 func (t *testDriver) awaitUpdatedServiceImport(serviceIP string) {
@@ -674,8 +863,8 @@ func (t *testDriver) awaitUpdatedServiceImport(serviceIP string) {
 	t.cluster2.awaitUpdatedServiceImport(t.service, serviceIP)
 }
 
-func (t *testDriver) awaitEndpointSlice(serviceIPs ...string) {
-	awaitEndpointSlice(t.brokerEndpointSliceClient, t.brokerServiceImportClient, t.endpoints, t.service, test.RemoteNamespace, false)
+func (t *testDriver) awaitEndpointSlice() {
+	t.awaitBrokerEndpointSlice()
 	t.cluster1.awaitEndpointSlice(t.endpoints, t.service)
 	t.cluster2.awaitEndpointSlice(t.endpoints, t.service)
 }
@@ -817,8 +1006,8 @@ func (t *testDriver) awaitNotServiceExportStatus(notCond *mcsv1a1.ServiceExportC
 }
 
 func (t *testDriver) awaitServiceExported(serviceIP string, statusIndex int) int {
-	t.awaitBrokerServiceImport(mcsv1a1.ClusterSetIP, serviceIP)
 	t.cluster1.awaitServiceImport(t.service, mcsv1a1.ClusterSetIP, serviceIP)
+	t.awaitBrokerServiceImport(mcsv1a1.ClusterSetIP, serviceIP)
 	t.cluster2.awaitServiceImport(t.service, mcsv1a1.ClusterSetIP, serviceIP)
 
 	t.awaitServiceExportStatus(statusIndex, newServiceExportCondition(mcsv1a1.ServiceExportValid,
@@ -843,9 +1032,6 @@ func (t *testDriver) awaitServiceUnexported() {
 func (t *testDriver) awaitHeadlessServiceUnexported() {
 	t.awaitServiceUnexported()
 
-	// In a real k8s env, the EndpointSlice would be deleted via k8s GC as it is owned by the ServiceImport.
-	Expect(t.cluster1.localEndpointSliceClient.Delete(t.endpoints.Name+"-"+clusterID1, nil)).To(Succeed())
-
 	t.awaitNoEndpointSlice(t.cluster1.localEndpointSliceClient)
 	t.awaitNoEndpointSlice(t.brokerEndpointSliceClient)
 	t.awaitNoEndpointSlice(t.cluster2.localEndpointSliceClient)
