@@ -1,3 +1,18 @@
+/*
+© 2020 Red Hat, Inc. and others
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+*/
 package lighthouse
 
 import (
@@ -13,7 +28,7 @@ import (
 const (
 	Svc        = "svc"
 	Pod        = "pod"
-	defaultTtl = uint32(5)
+	defaultTTL = uint32(5)
 )
 
 var (
@@ -22,7 +37,7 @@ var (
 
 // Define log to be a logger with the plugin name in it. This way we can just use log.Info and
 // friends to log.
-var log = clog.NewWithPlugin("lighthouse")
+var log = clog.NewWithPlugin(PluginName)
 
 type Lighthouse struct {
 	Next            plugin.Handler
@@ -47,7 +62,7 @@ type LocalServices interface {
 }
 
 type EndpointsStatus interface {
-	IsHealthy(name, namespace, clusterId string) bool
+	IsHealthy(name, namespace, clusterID string) bool
 }
 
 var _ plugin.Handler = &Lighthouse{}
